import java.awt.Canvas;
import java.awt.Color;
import java.awt.EventQueue;
import java.awt.Graphics;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.JTextField;
import java.awt.Toolkit;
import javax.swing.JLabel;
import javax.swing.JOptionPane;

import java.awt.Font;

/*                      Polynomial integration                 */
/*                      www.mosismath.com                      */

public class mainFrame {

	private JFrame frmPolynomialIntegration;
	private JPanel pGraph; 
	private JTextField tbIntegral;
	private JButton btnNewButton;
	
	int order = 10; // number of reference points
    int datapoints = 1000; // number of datapoints for the interpolation
   
    double integral = 0;
    double a = 0;
    double b = Math.PI / 2;
    public double deltaX;

    double[][] m;
    double[] y;
    double[] x;
    double[] yp = new double[datapoints];
    double[] xp = new double[datapoints];
    double[] y_k = new double[order + 1];
    double[] x_k = new double[order + 1];
    
    public class TPoint  {
		public float X;
        public float Y;	
	}
    
    public class myCanvas extends Canvas {
		
		private static final long serialVersionUID = 1L;

		public void paint(Graphics g) { 
			 			              
			  drawGraph(g, datapoints, a, b, true);
		}
    }
    
    private double f_x(double x)
    {
        return 5.0 / (Math.exp(Math.PI) - 2.0) * Math.exp(2 * x) * Math.cos(x);
    }

    public void Interpolate(int maxPoints, double maxTime)
    {
        int i;
        for (i = 0; i < maxPoints; i++)
        {
            yp[i] = 0;
            xp[i] = (double)(i) * maxTime / (double)maxPoints;
            yp[i] = f_x(xp[i]);
        }
    }
   
    
    public void drawGraph(Graphics g, int maxPoints, double minX, double maxX, boolean doClear)
    {
        TPoint p1, p2;
        int j;
        double maxTime = -1.0;
        double maxVal = -1.0;
        double minVal = 1.0;
        double scalefactor;
        boolean drawCeroline;
        for (j = 0; j < maxPoints - 1; j++)
        {
            if ((xp[j] >= minX) && (xp[j] <= maxX))
            {
                if (maxTime < xp[j])
                    maxTime = xp[j];
                if (maxVal < yp[j])
                    maxVal = yp[j];
                if (minVal > yp[j])
                    minVal = yp[j];
            }
        }
        maxTime = maxTime * 1.05;
        maxTime = maxTime * 1.05;
        if (minVal < 0)
        {
            if (maxVal > 0)
            {
                if (maxVal > Math.abs(minVal))
                    scalefactor = pGraph.getHeight() / maxVal / 2.2;
                else
                    scalefactor = pGraph.getHeight() / Math.abs(minVal) / 2.2;
            }
            else
                scalefactor = 5.0;
            drawCeroline = true;
        }
        else
        {
            scalefactor = pGraph.getHeight() / (maxVal) / 1.1;
            drawCeroline = false;
        }
        p1 = new TPoint();
        p2 = new TPoint();
        if (doClear)
        	g.clearRect(0, 0, pGraph.getWidth(), pGraph.getHeight());
       
        g.setColor(new Color(0, 0, 255));
        if (drawCeroline)
        {
            p1.X = 0;
            p1.Y = (pGraph.getHeight() / 2);
            p2.X = pGraph.getWidth();
            p2.Y = (pGraph.getHeight() / 2);
            g.setColor(new Color(0, 0, 0));
            g.drawLine((int)Math.round(p1.X),(int)Math.round(p1.Y),(int)Math.round( p2.X),(int)Math.round(p2.Y)); 
        }
        g.setColor(new Color(0, 0, 255));
        if (maxTime > 0) // draw interpolated graph
        {
            for (j = 0; j < maxPoints - 1; j++)
            {
                if ((xp[j] >= minX) && (xp[j] <= maxX))
                {
                    p1.X = (int)Math.round(xp[j] * pGraph.getWidth() / (double)maxTime);
                    p2.X = (int)Math.round(xp[j + 1] * pGraph.getWidth() / (double)maxTime);
                    if (drawCeroline)
                    {
                        p1.Y = (pGraph.getHeight() / 2) - (int)Math.round(yp[j] * scalefactor);
                        p2.Y = (pGraph.getHeight() / 2) - (int)Math.round(yp[j + 1] * scalefactor);
                    }
                    else
                    {
                        p1.Y = (pGraph.getHeight() - (int)Math.round(yp[j] * scalefactor));
                        p2.Y = (pGraph.getHeight() - (int)Math.round(yp[j + 1] * scalefactor));
                    }
                    g.drawLine((int)Math.round(p1.X),(int)Math.round(p1.Y),(int)Math.round( p2.X),(int)Math.round(p2.Y)); 
                }
            }
            g.setColor(new Color(255, 0, 0));
            for (j = 0; j < order; j++) // mark reference points
            {
                p1.X = (int)Math.round(x_k[j] * pGraph.getWidth() / (double)maxTime);
                if (drawCeroline)
                    p1.Y = (pGraph.getHeight() / 2) - (int)Math.round(y_k[j] * scalefactor);
                else
                    p1.Y = (pGraph.getHeight() - (int)Math.round(y_k[j] * scalefactor));
                g.drawOval((int)Math.round(p1.X-2), (int)Math.round(p1.Y-2), 4, 4);
            }
        }
    }
    
    private void calculate()
    {
        int i, k;
        deltaX = (b - a) / (double)order;
        for (i = 0; i < order; i++)
        {

            x[i] = 0.0;
        }

        // initialize supporting points
        i = 0;
        for (double x = a; x <= b; x += deltaX)
        {
            x_k[i] = x;
            y_k[i] = f_x(x);
            i++;
        }
       
        // fill matrix upside down
        for (i = 0; i < order; i++)
        {
            y[i] = y_k[order - 1 - i];
            for (k = 0; k < order; k++)
            {
                m[i][k] = Math.pow(x_k[order - 1 - i], (double)(k));
            }
        }

        TGauss gauss = new TGauss(order, m, y);

        if (gauss.Eliminate())
        {
            x = gauss.Solve();
            Interpolate(datapoints, b);

            integral = 0;
            // integraation
            for (i = 0; i < order; i++)
            {
                integral = integral + x[i] * (Math.pow(b, i + 1) - Math.pow(a, i + 1)) / (i + 1);
            }
            tbIntegral.setText(Double.toString((double)Math.round(100000*integral)/100000));
        }
        else
        	JOptionPane.showMessageDialog(frmPolynomialIntegration, "Matrix calculattion :-(");
    }

    /**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					mainFrame window = new mainFrame();
					window.frmPolynomialIntegration.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Create the application.
	 */
	public mainFrame() {
		initialize();
		m = new double[order][order];
        y = new double[order];
        x = new double[order];
	}

	/**
	 * Initialize the contents of the frame.
	 */
	private void initialize() {
		frmPolynomialIntegration = new JFrame();
		frmPolynomialIntegration.setTitle("Polynomial integration");
		frmPolynomialIntegration.setIconImage(Toolkit.getDefaultToolkit().getImage(mainFrame.class.getResource("/Ressource/Icon.png")));
		frmPolynomialIntegration.setBounds(100, 100, 697, 510);
		frmPolynomialIntegration.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frmPolynomialIntegration.getContentPane().setLayout(null);
		
		pGraph = new JPanel();
		final Canvas canvas = new myCanvas();
		canvas.setBackground(Color.WHITE);
		pGraph.addComponentListener(new ComponentAdapter() {
			@Override
			public void componentResized(ComponentEvent arg0) {
				canvas.setBounds(0, 0, pGraph.getWidth(), pGraph.getHeight());
			}
		});
		pGraph.setBackground(Color.WHITE);
		pGraph.setBounds(10, 11, 666, 393);
		frmPolynomialIntegration.getContentPane().add(pGraph);
		pGraph.add(canvas);
		pGraph.setLayout(null);
		
		btnNewButton = new JButton("New button");
		btnNewButton.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				calculate();
				canvas.repaint();
			}
		});
		
		btnNewButton.setBounds(539, 413, 132, 48);
		frmPolynomialIntegration.getContentPane().add(btnNewButton);
		
		tbIntegral = new JTextField();
		tbIntegral.setBounds(95, 422, 105, 30);
		frmPolynomialIntegration.getContentPane().add(tbIntegral);
		tbIntegral.setColumns(10);
		
		JLabel lblNewLabel = new JLabel("Integral");
		lblNewLabel.setFont(new Font("Tahoma", Font.PLAIN, 14));
		lblNewLabel.setBounds(20, 419, 78, 33);
		frmPolynomialIntegration.getContentPane().add(lblNewLabel);
		
	}
}
